/***************************************************************************
 *   Copyright (C) 2008 by Oliver Groß   *
 *   z.o.gross@gmx.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef QFTRANSCWORKER_H
#define QFTRANSCWORKER_H

#include <QObject>
#include <QTcpSocket>
#include <QDataStream>
#include "common.h"

namespace qftrans {
	/**
		@author Oliver Groß <z.o.gross@gmx.de>
	*/
	class CWorker : public QObject {
		Q_OBJECT
	public:
		CWorker(QTcpSocket * socket, QObject * parent = 0);
		~CWorker();
	signals:
		void uploadAdded(TransferData * transfer);
		void uploadRemoved(TransferData * transfer);
		void uploadUpdated(TransferData * transfer);
		
		void downloadAdded(TransferData * transfer);
		void downloadRemoved(TransferData * transfer);
		void downloadUpdated(TransferData * transfer);
	public slots:
		void addUpload(TransferData * transfer);
		void removeUpload(TransferData * transfer);
		void startUpload(TransferData * transfer);
		
		void removeDownload(TransferData * data);
		
		void setDestinationDir(QString & value);
		void writePing();
	protected slots:
		void writeData();
		void readData();
		
		void readPing();
	protected:
		void readHeader(QDataStream & in);
		
		void startUploadData(quint32 id);
		void sendAcknowledge(quint32 id);
		
		QTcpSocket * m_Socket;
		QString m_DestinationDir;
		
		quint32 m_IdCounter;
		QTransferIdHash m_UploadDict;
		
		QTransferDataHash m_DownloadDict;
		
		TransferHeader m_LastHeader;
		bool m_LastHeaderValid;
	private slots:
		void error(QAbstractSocket::SocketError socketError);
	};
}

#endif
